/*
AppleW.cpp							Copyright  2005 by George A. Nelson

By:  Tony Nelson  on  17 Mar 05	

Description:  Show Sk!11z.  In 1986 I wrote a Mac WDEF for Apple-Shaped Windows.
	Now I make simple Apple-Shaped Windows on Microsoft Windows.  This version
	uses a Window Region, as I already have experience with them.

	-- This one just does it to its own window.  I could be more invasive.
	
	-- Mouse events in the window caption aren't consistently handled by
	DefWindowProc().  Window dragging and left double-clicks to zoom the window
	are handled, but the close box, system menu, and right-clicks aren't handled.
	Seems to be just the usual Windows crappiness.  Probably don't have the
	proper events defined, either:  there needs to be one for drawing the close
	box (and other such things) hilited.

	-- My simulation of window caption mouse handling doesn't do the System menu
	properly.  A real system menu acts like a menu bar menu, and double-clicking
	activates the default command.  I can't do that, unless I do all the menu
	tracking myself.

	-- The System menu isn't getting updated "in time".  If I bring it up it
	doesn't reflect the current state.  If I close it by left-clicking outside
	it (but not if by right-clicking) it (usually?) shows the proper enabled
	state next time.  By doing the default processing on SysMenu left-clicks, I
	get the proper enabling.  I don't know how to trick the right-clicks into
	doing proper enabling.

	Magic:  Sending a SC_MOUSEMENU command, as DefWindowProc() would, causes the
	System menu to be properly enabled, even though it doesn't show.  Sending
	WM_INITMENU or WM_INITMENUPOPUP doesn't do it.  The magic must happen in
	SC_MOUSEMENU processing, without messages.  Also, it doesn't work the first
	time, and it doesn't work at all in the WM_NCRBUTTONUP code.
*/

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

//#define UNICODE -- done in project settings

#include "..\\Common\\CmnHdr.h"				// Richter _Programming Applications..._
#include <WindowsX.h>
#include <stdlib.h>
#include <tchar.h>
#include <math.h>

#include "resource.h"
#include "..\\Common\\Utility.h"
#include "..\\Common\\Hyperlinks.h"


#undef HANDLE_MSG
#define myHANDLE_MSG(hwnd, message, fn)    \
    case (message): lr = HANDLE_##message((hwnd), (wParam), (lParam), (fn)); break


/*	Globals
*/
HINSTANCE			hInst;
static HWND			gMainHwnd;

#define AWVNAME		_T("AppleShapedWindow")
typedef struct {
	HRGN			appleRgn, leafRgn, clientRgn;
	RECT			captionR, closeR,  sysMenuR;
	UINT			trackPart;
	bool			trackIn;
} AWV;
AWV					*awv;


/***********************  Apple Shaped stuff  **********************/

const int SizingBorderWidth = 4;

// Ellipse rects for window shape
//
static POINT const FrameData[] = {
	{    0,    0 }, { 862, 862 },			// first oval
	{  483,    0 },							// offset to second
	{    0, -377 }, {  938, -377+1620 },	// third oval
	{    0,  430 }, { 1343, 1243 },			// clip rect
	{  407,    0 },							// offset to fourth

	{ 1070,  150 }, { 1070+758, 150+758 },	// subtract the bite

	{  360, -430 }, { 360+483, -430+483 },	// intersect for leaf
	{  303,  183 }							// offset pt to second
};

//RECT const GoAwayBox =						// don't hilite this part of leaf
//	{ 360+303, -430+183,  360+303+483/2, -430+183+67 };
//RECT const GrowBox   =						// add grow box to structure
//	{ 567, 1100,  567+200, 1100+100 };
static RECT const FrameSize =				// scale down by this -- must be exact...
	{ 0, -233,  1272, 1242 };				// ...or window won't match its portRect


#define leftTop(r) ((POINT *) &r)[0]
#define rightBottom(r) ((POINT *) &r)[1]

	inline void
AddPoint( POINT & a, POINT const & b )
{
	a.x += b.x;
	a.y += b.y;
}

	inline void
SubPoint( POINT & a, POINT const & b )
{
	a.x -= b.x;
	a.y -= b.y;
}

	inline void
MulPoint( POINT & a, POINT const & b )
{
	a.x *= b.x;
	a.y *= b.y;
}

	inline void
DivPoint( POINT & a, POINT const & b )
{
	a.x /= b.x;
	a.y /= b.y;
}

	inline void
ScalePoint( POINT & p, POINT const & num, POINT const & denom )
{
	static POINT p22 = { 2, 2 };
	POINT d2 = denom;
	DivPoint( d2, p22 );

	MulPoint( p, num );
	AddPoint( p, d2 );
	DivPoint( p, denom );
}

	static void
MapPoints( POINT * pts, int nPts, RECT const & from, RECT const & to )
{
	POINT num = rightBottom( to );
	SubPoint( num, leftTop(to) );
	POINT denom = rightBottom( from );
	SubPoint( denom, leftTop(from) );

	for( int i = 0;  i < nPts;  ++i, ++pts ) {
		SubPoint( *pts, leftTop(from) );
		ScalePoint( *pts, num, denom );
		AddPoint( *pts, leftTop(to) );
	}
}


	static void
BuildOffset( POINT const * & pts, RECT const & wr, POINT & p )
{
	p = *pts++;
	p.y += FrameSize.top;
	MapPoints( &p, 1, FrameSize, wr );
}

	static void
BuildRect( POINT const * & pts, RECT const & wr, int inset, RECT & r )
{
	r = *((RECT *) pts);
	pts += 2;
	MapPoints( &leftTop(r), 2, FrameSize, wr );
	InflateRect( &r, -inset, -inset );
}

	static HRGN
BuildOvalRgn( POINT const * & pts, RECT const & wr, int inset )
{
	RECT r;
	BuildRect( pts, wr, inset, r );
	return CreateEllipticRgnIndirect( &r );
}

	static HRGN
BuildRectRgn( POINT const * & pts, RECT const & wr, int inset )
{
	RECT r;
	BuildRect( pts, wr, inset, r );
	return CreateRectRgnIndirect( &r );
}

	static void
MakeAppleRgns( RECT const & wr, int inset, HRGN * applep, HRGN * leafp )
{
	POINT const *fdp = FrameData;

	HRGN apple = CreateRectRgn( 0, 0, 0, 0 );
	HRGN rgn1 = BuildOvalRgn( fdp, wr, inset );		// first oval
	CopyRgn( apple, rgn1 );
	
	POINT p;
	BuildOffset( fdp, wr, p );						// offset to be second oval
	OffsetRgn( rgn1, p.x, p.y );
	int err = UnionRgn( apple, apple, rgn1 );
	
	HRGN rgn2 = BuildOvalRgn( fdp, wr, inset );		// third oval
	HRGN rgn3 = BuildRectRgn( fdp, wr, inset );		// clip rect
	err = IntersectRgn( rgn2, rgn2, rgn3 );			// half oval
	err = UnionRgn( apple, apple, rgn2 );

	BuildOffset( fdp, wr, p );						// offset to be fourth (half) oval
	OffsetRgn( rgn2, p.x, p.y );
	err = UnionRgn( apple, apple, rgn2 );

	HRGN rgn4 = BuildOvalRgn( fdp, wr, -inset );	// bite
	err = SubtractRgn( apple, apple, rgn4 );

	HRGN leaf = CreateRectRgn( 0, 0, 0, 0 );
	HRGN rgn5 = BuildOvalRgn( fdp, wr, inset );		// first leaf oval
	CopyRgn( leaf, rgn5 );

	BuildOffset( fdp, wr, p );						// offset to be second oval
	OffsetRgn( rgn5, p.x, p.y );
	err = IntersectRgn( leaf, leaf, rgn5 );

	DeleteRgn( rgn1 );
	DeleteRgn( rgn2 );
	DeleteRgn( rgn3 );
	DeleteRgn( rgn4 );
	DeleteRgn( rgn5 );

	*applep = apple;
	*leafp = leaf;
}


	static void
MakeWindowRgn( HWND hwnd )
{
	if( awv->appleRgn )
		DeleteRgn( awv->appleRgn );
	if( awv->leafRgn )
		DeleteRgn( awv->leafRgn );
	if( awv->clientRgn )
		DeleteRgn( awv->clientRgn );

	RECT cr;
	GetClientRect( hwnd, &cr );
	MakeAppleRgns( cr, 0, &awv->appleRgn, &awv->leafRgn );
	HRGN junkRgn;
	MakeAppleRgns( cr, (IsZoomed(hwnd)) ? 0 : SizingBorderWidth, &awv->clientRgn, &junkRgn );
	DeleteRgn( junkRgn );

	HRGN newWRgn = CreateRectRgn(0,0,0,0);
	int err = UnionRgn( newWRgn, awv->appleRgn, awv->leafRgn );
	err = err;
	RECT chr;
	GetChildRect( hwnd, hwnd, &chr );
	POINT p = leftTop( chr );
	OffsetRgn( newWRgn, -p.x, -p.y );

	HRGN oldWRgn = CreateRectRgn(0,0,0,0);
	BOOL ok = GetWindowRgn( hwnd, oldWRgn );
	ok = ok;

	if( !EqualRgn(newWRgn, oldWRgn) ) {
		int err = SetWindowRgn( hwnd, newWRgn, true );
		err = err;
		newWRgn = NULL;
	}
	
	if( newWRgn )
		DeleteRgn( newWRgn );

	GetRgnBox( awv->leafRgn, &awv->captionR );
	int boxh = awv->captionR.bottom - awv->captionR.top;

	awv->closeR = awv->captionR;
	awv->closeR.bottom = awv->closeR.top + boxh/4;

	awv->sysMenuR = awv->captionR;
	awv->sysMenuR.top = awv->sysMenuR.bottom - boxh/4;

	awv->captionR.top = awv->closeR.bottom;
	awv->captionR.bottom = awv->sysMenuR.top;
}


/*********************  Caption part tracking  *********************/


	static void
TrackDraw( HWND hwnd, UINT part )
{
	if( awv->trackPart == HTCLOSE ) {
		bool in = (part == awv->trackPart);
		if( in != awv->trackIn ) {
			HDC hdc = GetDC( hwnd );
			
			RECT r = awv->closeR;
			//GetPartRect( awv->trackPart, &r );
			HBRUSH hbr = CreateSolidBrush( (in) ? RGB(0xC0,0,0) : RGB(0xFF,0,0) );
			ExtSelectClipRgn( hdc, awv->leafRgn, RGN_AND );
			FillRect( hdc, &r, hbr );
			DeleteBrush( hbr );

			ReleaseDC( hwnd, hdc );
			awv->trackIn = in;
		}
	}
}


	inline bool
Tracking()
{
	return (awv->trackPart != 0);
}


	static void
TrackStart( HWND hwnd, UINT part )
{
	awv->trackPart = part;
	SetCapture( hwnd );
	TrackDraw( hwnd, part );
}


	static void
TrackMove( HWND hwnd, UINT part )
{
	if( Tracking() )
		TrackDraw( hwnd, part );
}


	static int /* part */
TrackFinish( UINT part, int * statusp )
{
	if( !Tracking() ) {
		*statusp = 0;
		return 0;
	}

	UINT trackPart = awv->trackPart;
	*statusp = (part == trackPart) ? 2 : 1;
	ReleaseCapture();
	awv->trackPart = 0;
	return trackPart;
}


/***************************  About Box  ***************************/

	static LRESULT CALLBACK
About( HWND hwnd, UINT message, WPARAM wParam, LPARAM lParam )
{
	switch (message)
	{
	case WM_INITDIALOG:
	{
		/*	Center on parent window apple body.
		*/
		RECT parentR;
		HWND parentW = GetParent( hwnd );
		AWV *awv = (AWV *) GetProp( parentW, AWVNAME );
		if( awv && awv->appleRgn )
			GetRgnBox( awv->appleRgn, &parentR );
		else
			GetClientRect( hwnd, &parentR );
		ClientToScreen( parentW, &leftTop(parentR) );
		ClientToScreen( parentW, &rightBottom(parentR) );
		CenterWindow( hwnd, parentR );

		/*	Set text.
		*/
		TCHAR text[4096];
		LoadString( hInst, IDS_ABOUT_TXT, text, chDIMOF(text) );
		HWND txtW = GetDlgItem( hwnd, IDC_TXT );
		SetWindowText( txtW, text );
		
		ConvertStaticToHyperlink( hwnd, IDC_EMAIL );

		/*	Make translucent.  Requires Win2K+.
		*/
		HMODULE user32H = GetModuleHandle( _T("user32.dll") );
		if( user32H ) {
			typedef WINUSERAPI BOOL WINAPI SetLayeredWindowAttributesT(
				HWND hwnd, COLORREF crKey, BYTE bAlpha, DWORD dwFlags);
			SetLayeredWindowAttributesT *SetLayeredWindowAttributesP = (SetLayeredWindowAttributesT *)
				GetProcAddress( user32H, "SetLayeredWindowAttributes" );

			if( SetLayeredWindowAttributesP ) {
				SetWindowLong( hwnd, GWL_EXSTYLE, 
					  GetWindowLong(hwnd, GWL_EXSTYLE) | WS_EX_LAYERED );

				BOOL ok = SetLayeredWindowAttributesP( hwnd, 0, 0xC0, LWA_ALPHA );
				ok = ok;
			}
		}

		return TRUE;
	}

	case WM_COMMAND:
		switch( LOWORD(wParam) )
		{
		case IDOK:
		case IDCANCEL:
			EndDialog(hwnd, LOWORD(wParam));
			return TRUE;

		case IDC_EMAIL:
			if( HIWORD(wParam) == STN_CLICKED ) {
				TCHAR str[256];
				LoadString( hInst, IDS_EMAIL, str, chDIMOF(str) );
				ShellExecute( hwnd, _T("open"), str, _T(""), _T(""), SW_SHOWNORMAL );
			}
			return TRUE;
		}
		break;
	}

    return FALSE;
}


/***************************  Main Wind  ***************************/


/*	The System Menu isn't automatically properly enabled when the window doesn't
	have its system menu in the expected place, so we have to do it ourselves. 
	I hope this is done right.  It should exactly mimic the normal enabling.

	Oddly, the menu is properly enabled after a WM_LBUTTONDOWN message (except
	for the first one?).

	Taken from TrayButton.cpp.
*/
	static void
EnableSystemMenu( HWND hwnd, HMENU hmenu )
{
	BOOL min    = IsIconic(hwnd);
	BOOL max    = !min && IsZoomed( hwnd );
	BOOL res    = !(min || max);
	DWORD style = GetWindowStyle( hwnd );

	UINT resable  = MF_ENABLED;
	UINT moveable = MF_GRAYED;
	UINT sizeable = MF_GRAYED;
	UINT minable  = (style & WS_MINIMIZEBOX) ? MF_ENABLED : MF_GRAYED;
	UINT maxable  = (style & WS_MAXIMIZEBOX) ? MF_ENABLED : MF_GRAYED;

	if( res ) {
		resable      = MF_GRAYED;
		moveable     = MF_ENABLED;
		if( style & WS_SIZEBOX )
			sizeable = MF_ENABLED;
	}
	else if( min )
		minable = MF_GRAYED;
	else if( max )
		maxable = MF_GRAYED;
	
	EnableMenuItem( hmenu, SC_RESTORE,  MF_BYCOMMAND | resable );
	EnableMenuItem( hmenu, SC_MOVE,     MF_BYCOMMAND | moveable  );
	EnableMenuItem( hmenu, SC_SIZE,     MF_BYCOMMAND | sizeable );
	EnableMenuItem( hmenu, SC_MINIMIZE, MF_BYCOMMAND | minable );			
	EnableMenuItem( hmenu, SC_MAXIMIZE, MF_BYCOMMAND | maxable );
	EnableMenuItem( hmenu, SC_CLOSE,    MF_BYCOMMAND | MF_ENABLED  );
	SetMenuDefaultItem( hmenu, SC_CLOSE, false );
	//pass to next WNDPROC
}


	static BOOL
Cls_OnCreate( HWND hwnd, LPCREATESTRUCT lpCreateStruct )
{
	awv = (AWV *) HeapAlloc( GetProcessHeap(), HEAP_ZERO_MEMORY, sizeof(AWV) );
	SetProp( hwnd, AWVNAME, awv );

	RECT r;
	SystemParametersInfo( SPI_GETWORKAREA, 0, &r, FALSE );
	CenterWindow( hwnd, r );
	MakeWindowRgn( hwnd );
	return true;
}


	static void
Cls_OnDestroy( HWND hwnd )
{
	if( awv->appleRgn )
		DeleteRgn( awv->appleRgn );
	if( awv->leafRgn )
		DeleteRgn( awv->leafRgn );
	if( awv->clientRgn )
		DeleteRgn( awv->clientRgn );
	awv = NULL;
	RemoveProp( hwnd, AWVNAME );

	gMainHwnd = NULL;
	PostQuitMessage( 0 );
}


	static void
Cls_OnPaint( HWND hwnd )
{
	PAINTSTRUCT ps;
	HDC hdc = BeginPaint( hwnd, &ps );

	HBRUSH redBrush    = CreateSolidBrush( RGB(0xFF,0,0) );
	HBRUSH greenBrush  = CreateSolidBrush( RGB(0,0xFF,0) );
	HBRUSH borderBrush = CreateSolidBrush( RGB(0xFF,0x80,0x80) );

	HBRUSH oldBrush = SelectBrush( hdc, borderBrush );
	PaintRgn( hdc, awv->appleRgn );
	SelectBrush( hdc, oldBrush );

	oldBrush = SelectBrush( hdc, redBrush );
	PaintRgn( hdc, awv->clientRgn );
	SelectBrush( hdc, oldBrush );

	ExtSelectClipRgn( hdc, awv->leafRgn, RGN_AND );

	FillRect( hdc, &awv->closeR,   redBrush );
	FillRect( hdc, &awv->captionR, greenBrush );
	FillRect( hdc, &awv->sysMenuR, GetStockBrush(WHITE_BRUSH) );
	FrameRgn( hdc, awv->leafRgn, borderBrush, 1, 1 );

	DeleteObject( greenBrush );
	DeleteObject( redBrush );
	DeleteObject( borderBrush );

	EndPaint( hwnd, &ps );
}


	static UINT
Cls_OnNCHitTest( HWND hwnd, int x, int y )
{
	UINT hit = HTNOWHERE;

	POINT at = { x, y };
	ScreenToClient( hwnd, &at );
	
	if( PtInRegion(awv->clientRgn, at.x, at.y) ) {
		hit = HTCLIENT;
	}
	else if( PtInRegion(awv->appleRgn, at.x, at.y) ) {
		RECT bbox;
		GetRgnBox( awv->appleRgn, &bbox );
		POINT ctr = { (bbox.left+bbox.right)/2, (bbox.top+bbox.bottom)/2 };
		POINT dif = at;
		SubPoint( dif, ctr );
		double angle = atan2( (double)dif.y, (double)dif.x );
		double pi2 = atan2( 1.0, 0.0 );
		int octant = ((int) floor( 2.0 * angle / pi2 + 4.5 )) & 0x7;

		switch( octant ) {
		case 0:		hit = HTLEFT;			break;
		case 1:		hit = HTTOPLEFT;		break;
		case 2:		hit = HTTOP;			break;
		case 3:		hit = HTTOPRIGHT;		break;
		case 4:		hit = HTRIGHT;			break;
		case 5:		hit = HTBOTTOMRIGHT;	break;
		case 6:		hit = HTBOTTOM;			break;
		case 7:		hit = HTBOTTOMLEFT;		break;
		}
	}
	else if( PtInRegion(awv->leafRgn, at.x, at.y) ) {
		hit = HTCAPTION;
		
		if( PtInRect(&awv->closeR, at) )
			hit = HTCLOSE;
		else if( PtInRect(&awv->sysMenuR, at) )
			hit = HTSYSMENU;
	}

	return hit;
}


	static void
Cls_OnNCLButtonDown( HWND hwnd, BOOL fDoubleClick, int x, int y, UINT codeHitTest )
{
	if( codeHitTest == HTCLOSE ) {
		TrackStart( hwnd, HTCLOSE );
		return;
	}
	else if( codeHitTest == HTSYSMENU ) {
		HMENU popM = GetSystemMenu( hwnd, FALSE );
		EnableSystemMenu( hwnd, popM );
		BOOL id = TrackPopupMenu( popM, TPM_RETURNCMD | TPM_NONOTIFY, x+2, y, 0, hwnd, NULL);
		if( id )
			SendMessage( hwnd, WM_SYSCOMMAND, id, 0xFFFF0000 );
		return;
	}

	FORWARD_WM_NCLBUTTONDOWN(hwnd, fDoubleClick, x, y, codeHitTest, DefWindowProc );
}

	static void
Cls_OnNCLButtonUp( HWND hwnd, int x, int y, UINT codeHitTest )
{
	if( Tracking() ) {
		int ts, tp = TrackFinish( codeHitTest, &ts );
		if( tp == HTCLOSE ) {
			switch( ts ) {
			case 2:
				SendMessage( hwnd, WM_SYSCOMMAND, SC_CLOSE, MAKELPARAM(x, y) );
				break;
			}
		}
		return;
	}

	FORWARD_WM_NCLBUTTONUP( hwnd, x, y, codeHitTest, DefWindowProc );
}


	static void
Cls_OnNCRButtonDown( HWND hwnd, BOOL fDoubleClick, int x, int y, UINT codeHitTest )
{
	if( codeHitTest == HTCAPTION  ||  codeHitTest == HTSYSMENU ) {
		TrackStart( hwnd, HTCAPTION );
		return;
	}

	FORWARD_WM_NCRBUTTONDOWN(hwnd, fDoubleClick, x, y, codeHitTest, DefWindowProc );
}


	static void
Cls_OnNCRButtonUp( HWND hwnd, int x, int y, UINT codeHitTest )
{
	/*	For some reason, a WM_NCRBUTTONDOWN message in the window caption is followed
		by a WM_RBUTTONUP instead of WM_NCRBUTTONUP.
	*/
	if( Tracking() ) {
		if( codeHitTest == HTSYSMENU )
			codeHitTest = HTCAPTION;
		int ts, tp = TrackFinish( codeHitTest, &ts );
		if( tp == HTCAPTION ) {
			switch( ts ) {
			case 2:
				HMENU popM = GetSystemMenu( hwnd, FALSE );
				EnableSystemMenu( hwnd, popM );
				BOOL id = TrackPopupMenu( popM, TPM_RIGHTBUTTON | TPM_RETURNCMD | TPM_NONOTIFY, x, y, 0, hwnd, NULL);
				if( id )
					SendMessage( hwnd, WM_SYSCOMMAND, id, 0xFFFF0000 );
				break;
			}
		}
		return;
	}

	FORWARD_WM_NCRBUTTONUP( hwnd, x, y, codeHitTest, DefWindowProc );
}


	static void
Cls_OnNCMouseMove( HWND hwnd, int x, int y, UINT codeHitTest )
{
	TrackMove( hwnd, codeHitTest );
}


	static void
Cls_OnLButtonDown( HWND hwnd, BOOL fDoubleClick, int x, int y, UINT keyFlags )
{
	DialogBox( hInst, MAKEINTRESOURCE(IDD_ABOUTBOX), hwnd, (DLGPROC) About );
}


	static void
Cls_OnLButtonUp( HWND hwnd, int x, int y, UINT keyFlags )
{
	if( Tracking() ) {
		POINT at = { x, y };
		ClientToScreen( hwnd, &at );
		UINT codeHitTest = Cls_OnNCHitTest( hwnd, at.x, at.y );
		int ts, tp = TrackFinish( codeHitTest, &ts );
		if( tp == HTCLOSE ) {
			switch( ts ) {
			case 2:
				SendMessage( hwnd, WM_SYSCOMMAND, SC_CLOSE, MAKELPARAM(x, y) );
				break;
			}
		}
		return;
	}

	FORWARD_WM_LBUTTONUP( hwnd, x, y, keyFlags, DefWindowProc );
}


	static void
Cls_OnRButtonUp( HWND hwnd, int x, int y, UINT flags )
{
	/*	For some reason, a WM_NCRBUTTONDOWN message in the window caption is followed
		by a WM_RBUTTONUP instead of WM_NCRBUTTONUP.
	*/
	if( Tracking() ) {
		POINT at = { x, y };
		ClientToScreen( hwnd, &at );
		UINT codeHitTest = Cls_OnNCHitTest( hwnd, at.x, at.y );
		if( codeHitTest == HTSYSMENU )
			codeHitTest = HTCAPTION;
		int ts, tp = TrackFinish( codeHitTest, &ts );
		if( tp == HTCAPTION ) {
			switch( ts ) {
			case 2:
				HMENU popM = GetSystemMenu( hwnd, FALSE );
				EnableSystemMenu( hwnd, popM );
				BOOL id = TrackPopupMenu( popM, TPM_RIGHTBUTTON | TPM_RETURNCMD | TPM_NONOTIFY, at.x, at.y, 0, hwnd, NULL);
				if( id )
					SendMessage( hwnd, WM_SYSCOMMAND, id, 0xFFFF0000 );
				break;
			}
		}
		return;
	}

	FORWARD_WM_RBUTTONUP( hwnd, x, y, flags, DefWindowProc );
}


	static void
Cls_OnMouseMove( HWND hwnd, int x, int y, UINT keyFlags )
{
	if( Tracking() ) {
		POINT at = { x, y };
		ClientToScreen( hwnd, &at );
		UINT codeHitTest = Cls_OnNCHitTest( hwnd, at.x, at.y );
		TrackMove( hwnd, codeHitTest );
		return;
	}

	FORWARD_WM_MOUSEMOVE(hwnd, x, y, keyFlags, DefWindowProc );
}


	static LRESULT CALLBACK
AppleWindProc( HWND hwnd, UINT uMsg, WPARAM wParam, LPARAM lParam )
{
	AWV *oldAwv = awv;
	awv = (AWV *) GetProp( hwnd, AWVNAME );
	LRESULT lr = 0;

	switch( uMsg )
	{
	default:
		lr = DefWindowProc( hwnd, uMsg, wParam, lParam );
		break;
	
	myHANDLE_MSG( hwnd, WM_CREATE,			Cls_OnCreate );
	myHANDLE_MSG( hwnd, WM_DESTROY,			Cls_OnDestroy );
	myHANDLE_MSG( hwnd, WM_PAINT,			Cls_OnPaint );
	myHANDLE_MSG( hwnd, WM_NCHITTEST,		Cls_OnNCHitTest );
	myHANDLE_MSG( hwnd, WM_NCLBUTTONDOWN,	Cls_OnNCLButtonDown );
	myHANDLE_MSG( hwnd, WM_NCLBUTTONUP,		Cls_OnNCLButtonUp );
	myHANDLE_MSG( hwnd, WM_NCLBUTTONDBLCLK,	Cls_OnNCLButtonDown );
	myHANDLE_MSG( hwnd, WM_NCRBUTTONDOWN,	Cls_OnNCRButtonDown );
	myHANDLE_MSG( hwnd, WM_NCRBUTTONUP,		Cls_OnNCRButtonUp );
	myHANDLE_MSG( hwnd, WM_NCMOUSEMOVE,		Cls_OnNCMouseMove );
	myHANDLE_MSG( hwnd, WM_LBUTTONDOWN,		Cls_OnLButtonDown );

	/*	During mouse capture, NC messages become regular ones?  Huh?
	*/
	myHANDLE_MSG( hwnd, WM_LBUTTONUP,		Cls_OnLButtonUp );
	myHANDLE_MSG( hwnd, WM_RBUTTONUP,		Cls_OnRButtonUp );
	myHANDLE_MSG( hwnd, WM_MOUSEMOVE,		Cls_OnMouseMove );
 
	case WM_SIZE:
		MakeWindowRgn( hwnd );
		break;

	case WM_CAPTURECHANGED:
		awv->trackPart = 0;
		break;

	//case WM_CONTEXTMENU:
	//case WM_NCRBUTTONDOWN:
	//case WM_NCRBUTTONUP:
	//case WM_NCLBUTTONDOWN:
	//case WM_NCLBUTTONDBLCLK:
	//case WM_RBUTTONUP:
	//case WM_LBUTTONUP:
	//case WM_LBUTTONDOWN:
	//case WM_LBUTTONDBLCLK:
		//MessageBeep( MB_OK );
		//break;
	}

	awv = oldAwv;
	return lr;
}


	int APIENTRY
_tWinMain( HINSTANCE hInstExe, HINSTANCE, LPSTR pszCmdLine, int iCmdShow )	//_txxx from <tchar.h>
{
	hInst = hInstExe;

	/*	AppleWindProc
	*/
	WNDCLASSEX	mwc = { 0 };
	mwc.cbSize			= sizeof mwc;
	mwc.style			= CS_HREDRAW | CS_VREDRAW;
	mwc.lpfnWndProc		= AppleWindProc;
	mwc.hInstance		= hInst;
	mwc.hIcon			= LoadIcon( hInst, MAKEINTRESOURCE(IDI_APPICON) );
	mwc.hCursor			= LoadCursor( NULL, IDC_ARROW );
	mwc.hbrBackground	= NULL;		//(HBRUSH) (COLOR_APPWORKSPACE + 1);
	mwc.lpszMenuName	= NULL;		//MAKEINTRESOURCE( IDC_MENU );
	mwc.lpszClassName	= TEXT("AppleWind");
	//mwc.hIconSm			= LoadIcon( hInst, MAKEINTRESOURCE(IDI_APPICON) );
	RegisterClassEx( &mwc );

	/*	Create window
	*/
	TCHAR title[64];
	LoadString( NULL, IDS_WTTL, title, chDIMOF(title) );

	gMainHwnd = CreateWindow( mwc.lpszClassName, title,
		WS_POPUP | WS_MINIMIZEBOX | WS_MAXIMIZEBOX | WS_SYSMENU | WS_THICKFRAME | WS_CLIPCHILDREN,
		CW_USEDEFAULT, CW_USEDEFAULT, 400, 428, //CW_USEDEFAULT, CW_USEDEFAULT,
		NULL, NULL, hInst, NULL );
	ShowWindow( gMainHwnd, iCmdShow );
	UpdateWindow( gMainHwnd );

	/*	Message loop
	*/
	MSG msg;
	while( GetMessage(&msg, NULL, 0, 0) ) {
		TranslateMessage( &msg );
		DispatchMessage( &msg );
	}

	return msg.wParam;
}
