;
;File defProcs/WDef.TEXT
;--------------------------------------------------------------------------
;
;  Window Definition Routine for the
;	MacIntosh Window Manager
;
;  written by Andy Hertzfeld  Aug 4, 1982
;
;  (c) 1982 by Apple Computer, Inc.  All rights reserved.
;
;    This file contains the window definition procedure
;  "DocumentProc", the standard Mac window type.  It is
;  a rectangular window with a title bar.  It is assembled
;  part of the window manager.
;
;  Modification History:
;
;    21-Aug-82	AJH  Made DocumentProc support rectangular windows only
;    30-Aug-82	AJH  Added growIcon hit detection, fixed hit to exclude perimeter drag
;    07-Sep-82	AJH  Added handler for grow message
;    12-Sep-82	AJH  Made it so proc doesn't have to calcRgns when not visible
;    20-Sep-82	AJH  Added go-away button drawing and hit-testing
;    25-Sep-82	AJH  Added cheap dialogBox window definition proc
;    28-Sep-82	AJH  New GoAway button; restructured goAway drawing
;    05-Oct-82	AJH  Removed Init message to save code
;    06-Oct-82	AJH  Fixed dialogBox dragRgn bug
;    10-Oct-82	AJH  Converted to QuickDraw trap interface
;    16-Oct-82	AJH  Fixed GoAway hit-test origin bug
;    17-OCt-82	AJH  Made both windowProcs preserve A1
;    07-Nov-82	AJH  Changed DocumentProc hiliting to Lisa way
;    14-Nov-82	AJH  Improved shape of Grow outline ala Lisa
;    16-Nov-82	AJH  Made branch table offset-based to save space
;    16-Nov-82	AJH  Special-cased hiliting of small windows
;    20-Dec-82	AJH  Changed title bar to 20 pixels tall
;    24-Dec-82	AJH  Made wProcAsm a separate assembly
;    17-Mar-83	AJH  Added third variant -- shadowless dBoxProc
;    27-Apr-83	AJH  only plot, hit-test goAway if window is active
;    25-Jul-83	SC   Fixed third variant -- frame was incorrect, see DrawDBox
;    06-Aug-83	AJH  changed hiliting to use pattern
;    09-Aug-83	AJH  added draw grow icon message receiver
;    20-Aug-83	AJH  made it only hit-test or draws goAway if window is active
;    29-Oct-83	AJH  new dBox border
;    10-Nov-83	AJH  added variant 4 -- just like variant 0, but no inGrow
;    24-Dec-83	AJH  fixed structRgn calc bug in variant 1
;    24-Feb-84	AJH  fixed hit-testing of go-away -- was off by 2
;    02-Feb-85	HAA  changed INCLUDEs to use new TLAsm file names
;    12-Apr-86	GAN  (George A. Nelson) finished conversion to MDS, added reference
;		     to BuildTBarRect, GoAwayMask, put grow icon in struct rgn.
;		     Message range test.
;    13-Apr-86	GAN  Apple shaped window
;
;----------------------------------------------------------------------------

; This file is set up for the MDS Assembler.
;
		INCLUDE	MacDefs.Txt
		INCLUDE	SysEquX.d
		INCLUDE	MacTraps.d
		INCLUDE	QuickEquX.d
		INCLUDE	ToolEquX.d
;
		resource 'WDEF' 0 'Struct rgn Grow box' 0 ; added 12-Apr-86 GAN


		.PROC	WDEF,0


;  FUNCTION   DocumentProc(selector: INTEGER;
;			   window: WindowPtr,
;			   message: INTEGER;
;			   parameter: LongInt): LongInt
;
DocProc
		BRA.S	DP1
;
; standard header
;
		.WORD	0			;flags word
		.ASCII	'WDEF'			;resource type
		.WORD	128			;resource ID
		.WORD	0			;version number
;
; boast
;
		dc.w	'    Apple window 14 April 1986 by George A. Nelson, 93 RiverView Ave., Waltham, MA  01254    '
		
		
DP1
		LINK	A6,#0			;set up a stack frame to address parameters
		MOVEM.L D3-D7/A1-A4,-(SP)	;save work registers
;
; always set WMgrPort -- Window manager forgets to, sometimes
;
		move.l	(a5),a0
		move.l	(a0),-(sp)
		move.l	WMgrPort,(a0)
;
; fetch the parameters into registers
;
		LEA	8(A6),A0		;get ptr to first parameter
		MOVE.L	(A0)+,D3		;get param in D3
		MOVE.W	(A0)+,D0		;get message
		MOVE.L	(A0)+,A3		;get the window pointer
		MOVE.W	(A0)+,D5		;get the selector integer
		MOVE	D5,D6			;keep copy in D6
		AND	#3,D5			;ignore "noGrow" variant
		CLR.L	(A0)			;clear out function result
;
; We need gobs of memory.  See if we have enough to work in.  The structRgn,
; for instance, can be about 3K, and we need several times that to make it.
; Not doing what we were told may be confusing, but it's better than crashing.
;
		cmp.w	#3,d0
		beq.s	@0
		tst.l	wDataHandle(a3)
		beq.s	noMem
@0
		move.l	d0,d4
		_FreeMem			;fast test for memory.  May be wildly wrong
		cmp.l	#20*1024,d0
		bge.s	@1
		move.l	#20*1024,d0		;slow test for memory
		_NewHandle
		bne.s	noMem
		_DisposHandle
@1
		move.l	d4,d0
;
; case out on the message number
;
		tst.w	d0
		bmi.s	badMsg
		cmp.w	#7,d0
		bge.s	badMsg
		
		ADD	D0,D0			;double for word index
		LEA	GoDocProc,A0		;get jump table address
		ADD.W	GODOCPROC(D0),A0	;compute dispatch address
		JSR	(A0)
badMsg
noMem
;
; we're done -- restore registers and return to caller
;
		_SetPort			;restore old port
		MOVEM.L (SP)+,D3-D7/A1-A4	;restore work registers
		UNLK	A6			;unlink stack frame
		MOVE.L	(SP)+,A0		;get return address
		ADD	#12,SP			;strip parameters
		JMP	(A0)			;return to caller
;
; DocumentProc dispatch table -- entries must be word branches!
;
GODOCPROC
		.WORD	DrawDoc-GoDocProc	;draw is message 0
		.WORD	HitDoc-GoDocProc	;hit test is message 1
		.WORD	CalcDoc-GoDocProc	;calc rgns is message 2
		.WORD	InitDoc-GoDocProc	;init is message 3
		.WORD	DisposeDoc-GoDocProc	;dispose is message 4
		.WORD	GrowDoc-GoDocProc	;grow message is #5
		.WORD	DrawGIcon-GoDocProc	;draw grow icon is #6
;
;  DrawDoc -- draw the document window.   The windowPtr is in A3
;
DrawDoc
		TST.B	WVISIBLE(A3)		;is it visible?
		BEQ	DoneDoc			;if not, don't do anything
;
; see if its a EORGoAway call and special case it
;
		CMP	#wInGoAway,D3		;is it a goAway call?
		BEQ	EorGoAway		;if so, go handle it
;
		_PenNormal			;we want the normal pen
;
; Frame the body of the window first.  _FrameRgn needs lots of memory.  It
; actually makes a new region that is the outline and _FillRgn's that.
;
DrawFrame
		MOVE.L	STRUCTRGN(A3),-(sp)	;get region handle
		_FrameRgn
		
		tst.w	d5			;dialog box?
		bne	DrawDBox
;
; draw the title bar, a leaf
;
		moveq	#1,d0
		bsr	MoveLeaf
		move.l	wDataHandle(a3),-(sp)
		
		moveq	#0,d0			;is it hilited?
		tst.b	WHilited(a3)
		beq.s	@0
		moveq	#8,d0			;hilited
@0
		pea	ttlPat(d0)
		
		move.l	StructRgn(a3),a0	;get topLeft
		move.l	(a0),a0
		move.l	RgnBBox+topLeft(a0),d0
		AND.L	#$00070007,D0		;only use mod 8
		MOVE.L	(A5),A0			;get grafGlobals
		lea	PatAlign(a0),a4
		move.l	d0,(a4)+		; set up patAlign
		neg.w	-(a4)			; negate it!  WDEF code was wrong!
		neg.w	-(a4)			; a4 points to PatAlign again
		
		_FillRgn			;fill the titleBar with the pattern
		
		clr.l	(a4)			;restore patAlign
;
; plot the goAway box if the window has one.
;
		tst.b	WGoAway(a3)		;is there a goAway button?
		beq.s	ReplaceLeaf		;if not, skip
		
		tst.b	WHilited(a3)		;only plot goAway if hilited
		beq.s	ReplaceLeaf		;if not, don't plot goAway
		
		bsr	BuildGoAway		;make goAway rgn
		clr.l	-(sp)
		_NewRgn
		move.l	(sp),d7
		pea	TempRect
		_RectRgn
		
		move.l	wDataHandle(a3),-(sp)	;erase only in leaf.  Can't use ClipRgn -- belongs...
		move.l	d7,-(sp)		;to the Window Manager; keeps us out of menu bar.
		move.l	d7,-(sp)
		_SectRgn
		
		move.l	d7,-(sp)
		_EraseRgn
		
		move.l	d7,-(sp)
		_DisposRgn
		
ReplaceLeaf
		moveq	#-1,d0
		bsr	MoveLeaf
;
; Draw the grow box if there is one.
;
		tst.w	d5			; document type window?
		bne.s	DoneDoc
		cmp.w	#4,D6			;inGrow enabled?
		beq.s	DoneDoc			;if not, skip
		bsr	DrawGIcon		; This may draw Grow more often than needed.
;
DoneDoc
		RTS

ttlPat
		dc.w	$0000,$0000,$0000,$0000
		dc.w	$FF00,$FF00,$FF00,$FF00
		

; Draw outline for dialog box (only variant 1).
;
DrawDBox
		CMP	#1,D5			;dialog box?
		BNE.S	DoneDoc			;if not, skip
		
		move.l	StructRgn(a3),-(sp)	;copy of structRgn for clearing
		clr.l	-(sp)
		_NewRgn
		move.l	(sp),d7
		_CopyRgn
		
		move.l	d7,-(sp)
		move.l	OneOne,-(sp)
		_InsetRgn
		
		move.l	d7,-(sp)		;find area between struct and content
		move.l	ContRgn(a3),-(sp)
		move.l	d7,-(sp)
		_DiffRgn
		
		move.l	d7,-(sp)		;paint it white
		move.l	(a5),a0
		pea	White(a0)
		_FillRgn
		
		move.l	ContRgn(a3),-(sp)	;copy of contRgn for framing aound
		move.l	d7,-(sp)
		_CopyRgn
		
		move.l	d7,-(sp)		;outset it by 5,5
		move.l	#$FFFBFFFB,-(sp)
		_InsetRgn
		
		move.l	StructRgn(a3),-(sp)	;stay inside frame!
		move.l	d7,-(sp)
		move.l	d7,-(sp)
		_SectRgn
		
		MOVE.L	(A5),A0
		PEA	Black(A0)
		_PenPat
		MOVE.L	#$00020002,-(SP)	;push new pen size
		_PenSize			;set new pen size
		move.l	d7,-(sp)
		_FrameRgn
		
		move.l	d7,-(sp)
		_DisposRgn

		_PenNormal
		
		bra	DoneDoc


; EORGoAway hilites/unhilites the goAway button.  It falls through into PlotGoAway
;
EorGoAway
		move.w	#patXor,-(sp)		;set EOR as the plot mode
		_PenMode
		move.l	(a5),a0
		pea	ltGray(a0)
		_PenPat
		
		moveq	#1,d0
		bsr	MoveLeaf
		
		bsr	BuildGoAway		;make goAway rgn
		clr.l	-(sp)
		_NewRgn
		move.l	(sp),d7
		pea	TempRect
		_RectRgn
		
		move.l	wDataHandle(a3),-(sp)	;draw only in leaf.  Can't use ClipRgn -- belongs...
		move.l	d7,-(sp)		;to the Window Manager; keeps us out of menu bar.
		move.l	d7,-(sp)
		_SectRgn
		
		move.l	d7,-(sp)
		_PaintRgn
		
		move.l	d7,-(sp)
		_DisposRgn
		
		moveq	#-1,d0
		bsr	MoveLeaf
		_PenNormal
		
		RTS


; HitDoc -- perform a hit test on the document.  On entry, D3 contains the
; mousePoint in global coordinates while A3 holds the window pointer
;
HitDoc
		CLR.W	-(SP)			;make room for function result
		MOVE.L	D3,-(SP)		;push the mouse point
		MOVE.L	CONTRGN(A3),-(SP)	;push content region handle
		_PtInRgn			;is the point in the content region?
		TST.B	(SP)+			;well, is it?
		BEQ.S	NotInContent		;if not, go check out drag region
		
		MOVEQ	#wInContent,D0		;return in content
		BRA.S	DoneHitDoc		;go store function result and return
;
; It's not in the content region -- see if it's in the drag region
;
NotInContent
		moveq	#1,d0
		bsr	MoveLeaf
		
		CLR.W	-(SP)			;make room for function result
		MOVE.L	D3,-(SP)		;push the mouse point
		MOVE.L	wDataHandle(A3),-(SP)	;push leaf region handle
		_PtInRgn			;is the point in the leaf region?
		move.b	(SP)+,d7		;save result
		
		moveq	#-1,d0
		bsr	MoveLeaf
		
		tst.b	d7
		BEQ.S	NotInDrag		;if not, return 0 (do nothing)
		
		bsr	BuildGoAway		;in leaf.  Is it in goAway?
		clr.w	-(sp)
		move.l	d3,-(sp)
		pea	TempRect
		_PtInRect
		tst.b	(sp)+
		beq.s	@0
		
		moveq	#wInGoAway,d0		;flag in goAway box
		bra.s	DoneHitDoc
@0
		MOVEQ	#wInDrag,D0		;flag in drag region
		bra.s	DoneHitDoc
;
; See if it's in grow region, whose rect is entirely contained within the
; structRgn.
;
NotInDrag
		bsr	BuildGrow
		clr.w	-(sp)
		move.l	d3,-(sp)
		pea	TempRect
		_PtInRect
		tst.b	(sp)+
		beq.s	DoneHit1
		
		moveq	#wInGrow,d0
		
DoneHitDoc
		MOVE.L	D0,20(A6)		;update function result
DoneHit1	RTS


; CalcDoc  -- calculate the structure and content regions for the window
; pointed to by A3.
;
CalcDoc
		lea	FrameData,a2
;
; Add ovals to form apple.
;
		bsr	BuildOvalRgn		;frame first oval
		move.l	a0,d7
		
		move.l	d7,-(sp)		;second oval is like it...
		clr.l	-(sp)
		_NewRgn
		move.l	(sp),d4
		_CopyRgn
		
		move.l	d4,-(sp)		;...but moved over a bit
		move.l	(a2)+,-(sp)
		pea	(sp)
		pea	FrameSize
		pea	PortRect(a3)
		_ScalePt
		_OfsetRgn
		
		move.l	d4,a0			;add second oval
		bsr	AddRgn
		
		bsr	BuildOvalRgn		;frame third oval
		move.l	a0,d4
		
		bsr	BuildRect		;clip it to the bottom half
		clr.l	-(sp)
		_NewRgn
		move.l	(sp),d3
		pea	TempRect
		_RectRgn
		move.l	d3,-(sp)
		move.l	d4,-(sp)
		move.l	d4,-(sp)
		_SectRgn
		move.l	d3,-(sp)		;done with rect rgn
		_DisposRgn
		
		move.l	d4,-(sp)		;add to apple
		move.l	d7,-(sp)
		move.l	d7,-(sp)
		_UnionRgn
		
		move.l	d4,-(sp)		;fourth oval is like it but moved over a bit
		move.l	(a2)+,-(sp)
		pea	(sp)
		pea	FrameSize
		pea	PortRect(a3)
		_ScalePt
		_OfsetRgn
		
		move.l	d4,a0			;add fourth oval
		bsr	AddRgn
;
; Remove bite from apple.
;
		bsr	BuildOvalRgn
		move.l	a0,d3
		
		move.l	d7,-(sp)		;subtract ellipse from shape
		move.l	d3,-(sp)
		move.l	d7,-(sp)
		_DiffRgn
		move.l	d3,-(sp)
		_DisposRgn
		
		move.l	d7,ContRgn(a3)		;save content region
;
; now do the structure region.  First add a bit at the flower area (grow
; box) to smooth it, then make the leaf, save it, and add it to the
; structure.  Outset the structure region by 1 for the frame.
;
		move.l	d7,-(sp)		;copy contRgn to structRgn
		clr.l	-(sp)
		_NewRgn
		move.l	(sp),d7
		_CopyRgn
		
		clr.l	-(sp)			;flower area is grow box
		_NewRgn
		move.l	(sp),d4
		bsr	BuildGrow
		pea	TempRect
		_RectRgn
		move.l	d4,a0
		bsr	AddRgn
		
		tst.w	d5			;dialog box?  gets no leaf
		beq.s	MakeLeaf
		
		cmp.w	#1,d5			;variant 1 gets border
		bne.s	StructCommon
		
		move.l	d7,-(sp)
		move.l	#$FFF9FFF9,-(sp)
		_InsetRgn
		bra.s	StructCommon
		
MakeLeaf
		bsr	BuildOvalRgn		;frame first leaf oval
		move.l	a0,d4
		
		move.l	d4,-(sp)		;second oval is like it...
		clr.l	-(sp)
		_NewRgn
		move.l	(sp),d3
		_CopyRgn
		
		move.l	d3,-(sp)		;...but moved over a bit
		move.l	(a2)+,-(sp)
		pea	(sp)
		pea	FrameSize
		pea	PortRect(a3)
		_ScalePt
		_OfsetRgn
		
		move.l	d4,-(sp)		;intersection is leaf
		move.l	d3,-(sp)
		move.l	d4,-(sp)
		_SectRgn
		move.l	d3,-(sp)
		_DisposRgn
		
		move.l	d4,-(sp)		;save leaf!
		move.l	wDataHandle(a3),-(sp)
		_CopyRgn
		moveq	#-1,d0			;tie it to window position
		bsr	MoveLeaf
		
		move.l	d4,a0			;add leaf to structure
		bsr	AddRgn
		
StructCommon
		move.l	d7,-(sp)
		MOVE.L	MinusOne,-(SP)		;make 1 pixel bigger, all around
		_InsetRgn			;make it bigger
		
		move.l	d7,structRgn(a3)	;save structure
		
		RTS


; Make an oval region.
;
;	a2 -- ptr to oval
;
; returns
;	a0 -- rgnHandle
;
BuildOvalRgn
		_OpenRgn
		bsr	BuildOval
		clr.l	-(sp)
		_NewRgn
		move.l	(sp),-(sp)
		_CloseRgn
		move.l	(sp)+,a0
		
		rts
		

; Add regions, a0 to d7.  dispose a0.
;
AddRgn
		pea	(a0)
		pea	(a0)
		move.l	d7,-(sp)
		move.l	d7,-(sp)
		_UnionRgn
		_DisposRgn
		
		rts
		

; Frame an oval, mapped to desired window rect
;
;	a2 -- ptr to oval to map and frame
;	a3 -- our window
;
; returns
;	a2 advanced by size of rect
;
BuildOval
		bsr	BuildRect
		pea	TempRect
		_FrameOval
		
		rts


; Build rect in TempRect mapped from our data to the desired window.
;
;	a2 -- ptr to rect to map
;	a3 -- our window
;
; returns
;	mapped rect in TempRect
;	a2 advanced by size of rect
;
BuildRect
		move.l	(a2)+,TempRect		;copy work rect
		move.l	(a2)+,TempRect+4
		
		move.l	PortRect+4(a3),-(sp)	;copy dest rect onto stack...
		move.l	PortRect(a3),-(sp)
		pea	(sp)			;...and offset to global coords
		move.l	PortBounds(a3),-(sp)
		neg.w	(sp)
		neg.w	2(sp)
		_OffsetRect
		
		move.l	sp,a0
		pea	TempRect
		pea	FrameSize
		pea	(a0)
		_MapRect
		
		addq.w	#8,sp			;pop dest rect
		rts
		

; Make the goAway box rect in TempRect.
;
BuildGoAway
		move.l	GoAwayBox,TempRect
		move.l	GoAwayBox+4,TempRect+4
		pea	TempRect
		pea	FrameSize
		move.l	ContRgn(a3),a0
		move.l	(a0),a0
		pea	RgnBBox(a0)		;_MapRect does no storage management....
		_MapRect
		
		move.w	TempRect+Bottom,d0	;make sure there is enough to touch
		sub.w	TempRect+Top,d0
		sub.w	#3,d0
		bge.s	@0
		sub.w	d0,TempRect+bottom
@0

		rts
		

; Make the grow box rect in TempRect.
;
BuildGrow
		move.l	GrowBox,TempRect
		move.l	GrowBox+4,TempRect+4
		pea	TempRect
		pea	FrameSize
		move.l	ContRgn(a3),a0
		move.l	(a0),a0
		pea	RgnBBox(a0)		;_MapRect does no storage management....
		_MapRect
		
		rts
		

; InitDoc puts a region, used for the leaf, in the dataHandle.
;
InitDoc
		clr.l	-(sp)
		_NewRgn
		move.l	(sp)+,wDataHandle(a3)
		
		rts
		

; DisposeDoc gets rid of the leaf
;
DisposeDoc
		move.l	wDataHandle(a3),-(sp)
		_DisposRgn
		
		rts
		

; GrowDoc handles the grow message by (slowly) drawing a grow outline of
; our apple based on the rectangle passed in D3.
;
GrowDoc
		move.l	ContRgn(a3),a0		;need copy of rect that won't move
		move.l	(a0),a0
		move.l	RgnBBox+topLeft(a0),TempRect+topLeft
		move.l	RgnBBox+botRight(a0),TempRect+botRight
		
		move.l	StructRgn(a3),-(sp)
		clr.l	-(sp)
		_NewRgn
		move.l	(sp),d7
		_CopyRgn
		
		move.l	d7,-(sp)		;structRgn copy
		pea	TempRect		;ContRgn bounds
		move.l	d3,-(sp)		;scale to this rect
		_MapRgn
		move.l	d7,-(sp)
		_FrameRgn			;frame it
		
		move.l	d7,-(sp)		;dispose structRgn copy
		_DisposRgn
		
		RTS				;all done!


;  Draw the grow icon in the structure region.  12-Apr-86 GAN
;
DrawGIcon
;
; Make the grow icon region.
;
		bsr	BuildGrow
		clr.l	-(sp)
		_NewRgn
		move.l	(sp),d7
		pea	TempRect
		_RectRgn
		
		move.l	d7,-(sp)
		move.l	ContRgn(a3),-(sp)
		move.l	d7,-(sp)
		_DiffRgn
;
; plot the grow box
;
		moveq	#0,d0			;assume not hilited
		TST.B	WHilited(A3)
		BEQ.S	@0
		moveq	#8,d0			;window is hilited
@0
		move.l	d7,-(sp)
		lea	ttlPat,a0
		pea	0(a0,d0.w)
		
		move.l	d7,a0			;get topLeft
		move.l	(a0),a0
		move.l	RgnBBox+topLeft(a0),d0
		AND.L	#$00070007,D0		;only use mod 8
		MOVE.L	(A5),A0			;get grafGlobals
		lea	PatAlign(a0),a4
		move.l	d0,(a4)+		; set up patAlign
		neg.w	-(a4)			; negate it!  WDEF code was wrong!
		neg.w	-(a4)			; a4 points to PatAlign again
		
		_FillRgn			;fill the titleBar with the pattern
		
		clr.l	(a4)
		
		move.l	d7,-(sp)
		_DisposRgn
;
; all done with DrawDocGrow
;
DoneDDG
		RTS
		

; Offset the leaf so it follows the window when the window is moved.
;
MoveLeaf
		move.l	wDataHandle(a3),-(sp)	;tie it to window position
		move.l	ContRgn(a3),a0
		move.l	(a0),a0
		move.l	RgnBBox+topLeft(a0),-(sp)
		tst.w	d0
		bpl.s	@0
		
		neg.w	(sp)
		neg.w	2(sp)
@0
		_OfsetRgn
		
		rts

		
; Ellipse rects for window shape
;
FrameData	dc.w	0,0, 862,862			;first oval
		dc.w	0,483				;offset pt to second
		dc.w	-377,0, -377+1622,938		;third oval
		dc.w	430,0, 1243,1343		;clip rect
		dc.w	0,407				;offset pt to fourth
		
		dc.w	140,1070, 140+758,1070+758	;subtract the bite
		
		dc.w	-430,360, -430+483,360+483	;intersect for leaf
		dc.w	183,303				;offset pt to second
		
GoAwayBox	dc.w	-430+183,360+303, -430+183+67,360+303+483/2	;don't hilite this part of leaf
GrowBox		dc.w	1100,567, 1100+100,567+200	;add grow box to structure
		
FrameSize	dc.w	0,0, 1243,1277			;scale down by this -- must be exact or...
							;...window won't match its portRect

		.END

